import {Component, HostListener, OnInit, Output, ViewChild, EventEmitter} from '@angular/core';
import {MatButton} from '@angular/material/button';
import {CprSearchEventService} from '../services/cpr-search-event.service';

@Component({
  selector: 'app-cpr-search-fab',
  templateUrl: './cpr-search-fab.component.html',
  styleUrls: ['./cpr-search-fab.component.scss']
})
export class CprSearchFabComponent implements OnInit {

  @ViewChild('cprSearchFAB') cprSearchFAB?: MatButton;

  @Output() cprSearchFABClickEvent = new EventEmitter<void>();

  public cprSearchFABVisible: boolean;

  constructor(private cprSearchEventService: CprSearchEventService) {
    this.cprSearchFABVisible = true;
  }

  ngOnInit(): void {
    this.cprSearchEventService.CprSearchFocused().subscribe((focused) => {
      this.setCprSearchFABDisabled(focused); // Disable button on focus / Enable on blur
    });
  }

  public onCprSearchFABClick(): void {
    this.cprSearchEventService.emitCprSearchFABClickEvent();
  }

  @HostListener('window:scroll')
  public allowYScrollFrame(): void {
    const supportPageOffset = window.pageXOffset !== undefined;
    const isCSS1Compat = ((document.compatMode || '') === 'CSS1Compat');
    const x = supportPageOffset ? window.pageXOffset : isCSS1Compat ? document.documentElement.scrollLeft : document.body.scrollLeft;
    const y = supportPageOffset ? window.pageYOffset : isCSS1Compat ? document.documentElement.scrollTop : document.body.scrollTop;
    const style = this.cprSearchFAB?._elementRef.nativeElement.style;
    if (style) {
      style.top = `${0}px`;
      style.left = `${-x}px`;
      style.position = `relative`;
      if (y >= 72) {
        style.position = `fixed`;
        style.top = `${10}px`;
        style.left = `${331 - x}px`;
      }
    }
  }

  private setCprSearchFABVisibility(visible: boolean): void {
    this.cprSearchFABVisible = visible;
  }

  private setCprSearchFABDisabled(disabled: boolean): void {
    if (this.cprSearchFAB) {
      this.cprSearchFAB.disabled = disabled;
    }
  }
}
