import {NgModule} from '@angular/core';
import {RouterModule, Routes} from '@angular/router';
import {MeasurementsTabComponent} from '../measurements/measurements-tab/measurements-tab.component';
import {ActivitiesComponent} from '../activities/activities.component';
import {PregnancyComponent} from '../pregnancy/pregnancy.component';
import {PatientInformationComponent} from '../patient-information/patient-information.component';
import {DirtyGuard} from '../confirm-navigation-if-dirty.framework';
import {TriageOverviewComponent} from '../triage-overview/triage-overview.component';
import {PrivatelyMarkedComponent} from '../privately-marked/privately-marked.component';
import {PrivatelyMarkedGuard} from '../privately-marked/privately-marked.guard';
import {NegatePrivatelyMarkedGuard} from '../privately-marked/negate-privately-marked.guard';
import {SomethingWentWrongComponent} from '../something-went-wrong/something-went-wrong.component';
import {CprDoesNotExistComponent} from '../cpr-does-not-exist/cpr-does-not-exist.component';


/**
 * The application routing parameters
 */
export const appRoutes: Routes = [
  { path: 'overblik-til-visitation',
    component: TriageOverviewComponent,
    canActivate: [PrivatelyMarkedGuard],
    canDeactivate: [DirtyGuard],
    runGuardsAndResolvers: 'always',
  },
  { path: 'forløbsplan',
    component: ActivitiesComponent,
    canActivate: [PrivatelyMarkedGuard],
    canDeactivate: [DirtyGuard],
    runGuardsAndResolvers: 'always',
  },
  { path: 'undersøgelser-og-målinger',
    component: MeasurementsTabComponent,
    canActivate: [PrivatelyMarkedGuard],
    canDeactivate: [DirtyGuard],
    runGuardsAndResolvers: 'always',
  },
  { path: 'svangerskab',
    component: PregnancyComponent,
    canActivate: [PrivatelyMarkedGuard],
    canDeactivate: [DirtyGuard],
    runGuardsAndResolvers: 'always',
  },
  { path: 'patientoplysninger',
    component: PatientInformationComponent,
    canActivate: [PrivatelyMarkedGuard],
    canDeactivate: [DirtyGuard],
    runGuardsAndResolvers: 'always',
  },
  { path: 'privatmarkerede-oplysninger',
    component: PrivatelyMarkedComponent,
    canActivate: [NegatePrivatelyMarkedGuard], // TODO: Enable again when '/privatmarkerede-oplysninger' is done!
    canDeactivate: [DirtyGuard],
    runGuardsAndResolvers: 'always',
  },
  { path: 'der-skete-en-fejl',
    component: SomethingWentWrongComponent,
    canActivate: [],
    canDeactivate: [],
    runGuardsAndResolvers: 'always',
  },
  { path: 'cpr-findes-ikke',
    component: CprDoesNotExistComponent,
    canActivate: [],
    canDeactivate: [],
    runGuardsAndResolvers: 'always',
  },
  { path: '**',
    redirectTo: 'forløbsplan',
    canActivate: [PrivatelyMarkedGuard],
    canDeactivate: [DirtyGuard],
    runGuardsAndResolvers: 'always',
  }
];

@NgModule({
  imports: [RouterModule.forRoot(appRoutes, {useHash: true, onSameUrlNavigation: 'reload'})],
  exports: [RouterModule]
})

export class RoutingModule { }
