import {ComponentFixture, fakeAsync, TestBed, tick} from '@angular/core/testing';

import { PrivatelyMarkedComponent } from './privately-marked.component';
import {AngularMaterialModule} from '../modules/angular-material.module';
import {DialogService} from '../modals/dialog/dialog.service';
import {of} from 'rxjs/internal/observable/of';
import {Observable} from 'rxjs';
import {PatientContextService} from '../services/patient-context.service';
import {TemplateRef} from '@angular/core';

describe('PrivatelyMarkedComponent', () => {
  let component: PrivatelyMarkedComponent;
  let fixture: ComponentFixture<PrivatelyMarkedComponent>;
  let mockDialog: TemplateRef<any>;
  let dialogService: DialogService;
  let dialogObservable: Observable<boolean>;
  let patientContextService: PatientContextService;

  beforeEach(async () => {
    dialogService = {
      show: () => dialogObservable
    } as any;

    patientContextService = {
      setXBreakTheGlassReason: () => {}
    } as any;

    mockDialog = {
    } as any;

    await TestBed.configureTestingModule({
      declarations: [ PrivatelyMarkedComponent ],
      imports: [AngularMaterialModule],
      providers: [
        {provide: DialogService, useValue: dialogService},
        {provide: PatientContextService, useValue: patientContextService},
      ],
    }).compileComponents();
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(PrivatelyMarkedComponent);
    component = fixture.componentInstance;
    component.reloadPage = () => {};
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('when the open dialog button is clicked', () => {
    it('should call openDialog', fakeAsync(() => {
      spyOn(component, 'openDialog');

      const button = fixture.debugElement.nativeElement.querySelector('button');
      button.click();
      tick();
      expect(component.openDialog).toHaveBeenCalled();
    }));
  });

  describe('when dialog closes with "yes" option', () => {
    beforeEach(() => {
      dialogObservable = of(true);
    });

    it('should set patientContext.XBreakTheGlassReason', () => {
      spyOn(patientContextService, 'setXBreakTheGlassReason');
      component.openDialog();
      expect(patientContextService.setXBreakTheGlassReason).toHaveBeenCalledTimes(1);
    });

    it('should reload the page', () => {
      spyOn(component, 'reloadPage');
      component.openDialog();
      expect(component.reloadPage).toHaveBeenCalledTimes(1);
    });
  });

  describe('when dialog closes with "no" option', () => {
    beforeEach(() => {
      dialogObservable = of(false);
    });

    it('should not set patientContext.XBreakTheGlassReason', () => {
      spyOn(patientContextService, 'setXBreakTheGlassReason');
      component.openDialog();
      expect(patientContextService.setXBreakTheGlassReason).toHaveBeenCalledTimes(0);
    });

    it('should not reload the page', () => {
      spyOn(component, 'reloadPage');
      component.openDialog();
      expect(component.reloadPage).toHaveBeenCalledTimes(0);
    });
  });

});
