import {Injectable} from '@angular/core';
import {CanActivate, ActivatedRouteSnapshot, RouterStateSnapshot, Router} from '@angular/router';
import {PatientContextService} from '../services/patient-context.service';

@Injectable({
  providedIn: 'root'
})
export class PrivatelyMarkedGuard implements CanActivate {
  constructor(private router: Router,
              private patientContextService: PatientContextService) {
  }

  canActivate(route: ActivatedRouteSnapshot, state: RouterStateSnapshot): boolean {
    const canAccessData =
      !!this.patientContextService.getCurrentXBreakTheGlassReason()
      || !this.patientContextService.getCurrentHasPrivatelyMarkedData();

    if (!canAccessData) {
      const privatelyMarkedUrl = '/privatmarkerede-oplysninger';
      // If we didn't come from privatelyMarkedUrl then redirect. Avoids infinite-loop in negate-privately-marked.guard
      if (!state.url.includes(privatelyMarkedUrl)) {
        this.router.navigate([privatelyMarkedUrl]);
      }
      return false;
    }
    return true;
  }
}
