import {ErrorHandler, Injectable, NgZone} from '@angular/core';
import {Router} from '@angular/router';
import {PatientContextService} from './patient-context.service';

@Injectable({
  providedIn: 'root'
})
export class ErrorService implements ErrorHandler {

  constructor(private zone: NgZone,
              private router: Router,
              private patientContextService: PatientContextService) {
  }

  handleError(error: any): void {
    if (error instanceof CprDoesNotExistError) {
      this.patientContextService.setPatientExists(false);
    } else if (error instanceof AppError) {
      this.zone.run(() => this.router.navigate(['/der-skete-en-fejl']));
    }
    console.error(error);
  }
}

// Custom App Errors
/**
 * The Generic Application error. Parent to all other errors.
 */
export interface GenericAppError {
  errorMessage: string;
  innerError?: Error;
}

/**
 * Top level error
 */
export class AppError implements GenericAppError {
  errorMessage = 'Der skete en ukendt fejl';
  constructor(public innerError?: Error) {
  }
}

/**
 * Expected error for 404
 */
export class PageNotFoundError extends AppError {
  errorMessage = 'Siden kunne ikke findes';
}

/**
 * Expected error for when patient does not exist
 */
export class CprDoesNotExistError extends AppError {
  errorMessage = 'Cpr findes ikke';
}
