import {HttpErrorResponse} from '@angular/common/http';
import {Observable, throwError} from 'rxjs';
import {environment} from '../environments/environment';
import {deleteCookie} from './cookies';
import {AppError, CprDoesNotExistError, PageNotFoundError} from '../app/services/error.service';


/**
 * Specific app error handler
 * @param error HttpErrorResponse
 */
export function handleError(error: HttpErrorResponse): Observable<never> {
  // FIXME: This should be a real service so we can inject dialogService to tell users something is wrong
  console.log('handleError: status: ' + error.status + ' message: ' + error.message);
  redirectIfRequested(error);
  if (error.status === 404) {
    return throwError(new PageNotFoundError(error));
  }
  if (error.status === 400 ) {
    return throwError(new CprDoesNotExistError(error));
  }
  return throwError(new AppError(error));
}

/**
 * Browser intercepts redirects in xhr calls and returns 200 OK
 * this will check if any login redirects were requested by SAML
 * @param error the httpErrorResponse
 */
function redirectIfRequested(error: HttpErrorResponse): void {
  const redirectHtmlStart = '<html><head><meta http-equiv="refresh" content=';
  // Regex for oiosaml.fragment Cookie: url=(.*?)\".*document\.cookie = '(.*)';

  if (error.status === 200 && error.error.text?.includes(redirectHtmlStart)) {
    console.log('should be redirected');
    const redirectUrlRegex = /url=(.*?)\"/;
    const matches = redirectUrlRegex.exec(error.error.text);
    if (matches !== null && matches.length > 1) {
      deleteCookie('JSESSIONID');
      window.location.assign(environment.base_path);
    }
  }
}
